@@ -2,16 +2,12 @@
 """
 
 
-__version__ = "1.3.3"
+__version__ = "1.4.1"
 __author__ = "Yngve Mardal Moe"
 __email__ = "yngve.m.moe@gmail.com"
 
 
-from group_lasso._group_lasso import (
-    GroupLasso,
-    LogisticGroupLasso,
-    BaseGroupLasso,
-)
-
+from group_lasso._group_lasso import (BaseGroupLasso, GroupLasso,
+                                      LogisticGroupLasso)
 
 MultinomialGroupLasso = LogisticGroupLasso
